

import userdb.*;

import java.sql.*;
import org.omg.CosTransactions.*;


public class UnixUserResourceImpl extends UnixUserResourcePOA
{

    Coordinator myCoordinator;
    UnixUserServant myServant;
    UnixUserStatus n;
    Connection db;

    boolean errorOccurred;


    public UnixUserStatus newStatus()
    {
	return n;
    }

    public void newStatus(UnixUserStatus s)
    {
	n = s;
    }

    Coordinator coordinator()
    {
	return myCoordinator;
    }

    public UnixUserResourceImpl (UnixUserServant s, Coordinator c)
    {
	myServant = s;
	myCoordinator = c;
	n = null;
	errorOccurred = false;
	db = null;
    }


    public Vote prepare()
    {
	System.out.println("TRANSACTION prepare");

	db_update();

	if (errorOccurred)
	{
	    db_rollback();
	    myServant.removeResource(this);
	    return Vote.VoteRollback;
	}
	else
	{
	    return Vote.VoteCommit;
	}
    }

    public void commit_one_phase()
    {
	System.out.println("TRANSACTION commit_one_phase");

	db_update();
	db_commit();

	myServant.removeResource(this);
    }

    public void commit()
    {
	System.out.println("TRANSACTION commit");

	db_commit();

	myServant.removeResource(this);
    }

    public void forget()
    {
	System.out.println("TRANSACTION forget");
	myServant.removeResource(this);
    }

    public void rollback()
    {
	System.out.println("TRANSACTION rollback");

	db_rollback();

	myServant.removeResource(this);
    }


    void db_update()
    {
	System.out.println(" DB update");
	try
	{
	    db = DriverManager.getConnection(myServant.dbUrl,
		myServant.dbUser, myServant.dbPassword);

	    Statement stmt = db.createStatement();

	    db.setAutoCommit(false);


            int status = 0;
            
            if (n.equals(UnixUserStatus.NEW))
            {
                status = 1;
            }   
            else if (n.equals(UnixUserStatus.DENY_ACCESS))
            {
                status = 9;
            }   
            else if (n.equals(UnixUserStatus.READY))
            {
                status = 10;
            }

	    int ret = stmt.executeUpdate(
		 "UPDATE USERIDS SET status=" + status +
		   "WHERE sid='" + myServant.state.sid + "'");
		    
	    if (ret != 1)
	    {
	       System.out.println("update failed");
	       errorOccurred = true;
	    }

	    stmt.close();
	}
	catch (Exception e)
	{
	    e.printStackTrace();
	    errorOccurred = true;
	}
    }

    void db_commit()
    {
	System.out.println(" DB commit");
	try
	{
	    db.commit();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }

    void db_rollback()
    {
	System.out.println(" DB rollback");
	try
	{
	    if (db != null)
	    {
		db.rollback();
	    }
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }

}
